@@ -1,5 +1,5 @@
  /* 
-  Copyright (c) 2005, 2006 Rocky Bernstein <rocky@panix.com>
+  Copyright (c) 2005, 2006 Rocky Bernstein <rocky@cpan.org>
   Copyright (c) 2001-2002  Ben Fennema <bfennema@falcon.csc.calpoly.edu>
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -67,12 +67,6 @@ typedef enum {
   UDF_VOLSET_ID_SIZE =  128
 } ecma_167_enum1_t ;
 
-/** This variable is trickery to force the above enum symbol values to
-    be recorded in debug symbol tables. It is used to allow one refer
-    to above enumeration values in a debugger and debugger
-    expressions */
-extern ecma_167_enum1_t debug_ecma_167_enums1;
-
 /** Tag Identifier (ECMA 167r3 3/7.2.1) */
 
 typedef enum {
@@ -161,12 +155,6 @@ typedef enum {
   TIMESTAMP_TIMEZONE_MASK   =	0x0FFF,
 } ecma_167_timezone_enum_t ;
 
-/** This variable is trickery to force the above enum symbol values to
-    be recorded in debug symbol tables. It is used to allow one refer
-    to above enumeration values in a debugger and debugger
-    expressions */
-extern ecma_167_timezone_enum_t debug_ecma_167_timezone_enum;
-
 
 #define TIMESTAMP_TYPE_MASK		0xF000
 #define TIMESTAMP_TYPE_CUT		0x0000
@@ -174,12 +162,22 @@ extern ecma_167_timezone_enum_t debug_ecma_167_timezone_enum;
 #define TIMESTAMP_TYPE_AGREEMENT	0x2000
 #define TIMESTAMP_TIMEZONE_MASK		0x0FFF
 
+struct udf_id_suffix_s
+{
+  udf_Uint16_t  udf_revision;
+  udf_Uint8_t   os_class;
+  udf_Uint8_t   os_identifier;
+  udf_Uint8_t   reserved[4];
+} GNUC_PACKED;
+
+typedef struct udf_id_suffix_s udf_id_suffix_t;
+
 /** Entity identifier (ECMA 167r3 1/7.4) */
 struct udf_regid_s
 {
   udf_Uint8_t		flags;
   udf_Uint8_t		id[UDF_REGID_ID_SIZE];
-  udf_Uint8_t		id_suffix[8];
+  udf_id_suffix_t	id_suffix;
 } GNUC_PACKED;
 
 typedef struct udf_regid_s udf_regid_t;
@@ -620,12 +618,16 @@ typedef struct udf_icbtag_s udf_icbtag_t;
 #define	UDF_ICB_TAG_FLAGS_SETGID	0x80
 #define	UDF_ICB_TAG_FLAGS_STICKY	0x100
 
-/** Strategy Type (ECMA 167r3 4/14.6.2) */
-#define ICBTAG_STRATEGY_TYPE_UNDEF	0x0000
-#define ICBTAG_STRATEGY_TYPE_1		0x0001
-#define ICBTAG_STRATEGY_TYPE_2		0x0002
-#define ICBTAG_STRATEGY_TYPE_3		0x0003
-#define ICBTAG_STRATEGY_TYPE_4		0x0004
+/** Strategy Type (ECMA 167r3 4/14.6.2) which helpfully points
+    largely to 4/A.x */
+#define ICBTAG_STRATEGY_TYPE_UNDEF 0x0000 
+#define ICBTAG_STRATEGY_TYPE_1	   0x0001 /**< 4/A.2 Direct entries Uint16 */
+#define ICBTAG_STRATEGY_TYPE_2	   0x0002 /**< 4/A.3 List of ICB direct entries */
+#define ICBTAG_STRATEGY_TYPE_3	   0x0003 /**< 4/A.4 */
+#define ICBTAG_STRATEGY_TYPE_4	   0x0004 /**< 4/A.5 Hierarchy having one
+					   single ICB with one direct entry.
+					   This is what's most often used.
+					  */
 
 /** File Type (ECMA 167r3 4/14.6.6) 
 
@@ -649,30 +651,40 @@ typedef enum {
   ICBTAG_FILE_TYPE_STREAMDIR =	0x0D
 } icbtag_file_type_enum_t;
 
-/** This variable is trickery to force the above enum symbol values to
-    be recorded in debug symbol tables. It is used to allow one refer
-    to above enumeration values in a debugger and debugger
-    expressions */
-extern icbtag_file_type_enum_t debug_icbtag_file_type_enum;
-
 /** Flags (ECMA 167r3 4/14.6.8) */
-#define ICBTAG_FLAG_AD_MASK		0x0007
-#define ICBTAG_FLAG_AD_SHORT		0x0000
-#define ICBTAG_FLAG_AD_LONG		0x0001
-#define ICBTAG_FLAG_AD_EXTENDED		0x0002
-#define ICBTAG_FLAG_AD_IN_ICB		0x0003
-#define ICBTAG_FLAG_SORTED		0x0008
-#define ICBTAG_FLAG_NONRELOCATABLE	0x0010
-#define ICBTAG_FLAG_ARCHIVE		0x0020
-#define ICBTAG_FLAG_SETUID		0x0040
-#define ICBTAG_FLAG_SETGID		0x0080
-#define ICBTAG_FLAG_STICKY		0x0100
-#define ICBTAG_FLAG_CONTIGUOUS		0x0200
-#define ICBTAG_FLAG_SYSTEM		0x0400
-#define ICBTAG_FLAG_TRANSFORMED		0x0800
-#define ICBTAG_FLAG_MULTIVERSIONS	0x1000
-#define ICBTAG_FLAG_STREAM		0x2000
-
+typedef enum {
+  ICBTAG_FLAG_AD_MASK        =	0x0007, /**< "&" this to get below address
+					     flags */
+  ICBTAG_FLAG_AD_SHORT       =	0x0000, /**< The allocation descriptor
+					     field is filled with
+					     short_ad's.  If the
+					     offset is beyond the
+					     current extent, look for
+					     the next extent. */
+  ICBTAG_FLAG_AD_LONG        =	0x0001, /**< The allocation descriptor
+					     field is filled with
+					     long_ad's If the offset
+					     is beyond the current
+					     extent, look for the next
+					     extent. */
+  ICBTAG_FLAG_AD_EXTENDED    =	0x0002,
+  ICBTAG_FLAG_AD_IN_ICB      =	0x0003, /**< This type means that the
+					     file *data* is stored in
+					     the allocation descriptor
+					     field of the file entry. */
+  ICBTAG_FLAG_SORTED         =	0x0008,
+  ICBTAG_FLAG_NONRELOCATABLE =	0x0010,
+  ICBTAG_FLAG_ARCHIVE        =	0x0020,
+  ICBTAG_FLAG_SETUID         =	0x0040,
+  ICBTAG_FLAG_SETGID         =	0x0080,
+  ICBTAG_FLAG_STICKY         =	0x0100,
+  ICBTAG_FLAG_CONTIGUOUS     =	0x0200,
+  ICBTAG_FLAG_SYSTEM         =	0x0400,
+  ICBTAG_FLAG_TRANSFORMED    =	0x0800,
+  ICBTAG_FLAG_MULTIVERSIONS  =	0x1000,
+  ICBTAG_FLAG_STREAM =		0x2000
+} icbtag_flag_enum_t;
+  
 /** Indirect Entry (ECMA 167r3 4/14.7) */
 struct indirect_entry_s
 {
@@ -712,7 +724,7 @@ struct udf_file_entry_s
   udf_Uint32_t	  checkpoint;
   udf_long_ad_t	  ext_attr_ICB;
   udf_regid_t	  imp_id;
-  udf_Uint64_t	  unique_iD;
+  udf_Uint64_t	  unique_ID;
   udf_Uint32_t	  i_extended_attr;
   udf_Uint32_t	  i_alloc_descs;
   udf_Uint8_t	  ext_attr[0];
@@ -984,8 +996,11 @@ PRAGMA_END_PACKED
     allow one refer to the enumeration value names in the typedefs
     above in a debugger and in debugger expressions.
 */
-extern file_characteristics_t debug_file_characteristics;
-extern tag_id_t debug_tagid;
-
-
+extern tag_id_t                 debug_tagid;
+extern file_characteristics_t   debug_file_characteristics;
+extern icbtag_file_type_enum_t  debug_icbtag_file_type_enum;
+extern icbtag_flag_enum_t       debug_flag_enum;
+extern ecma_167_enum1_t         debug_ecma_167_enum1;
+extern ecma_167_timezone_enum_t debug_ecma_167_timezone_enum;
+  
 #endif /* _ECMA_167_H */
