@@ -1,8 +1,8 @@
 /*
-    $Id: iso9660_fs.c,v 1.37 2006/09/26 22:21:09 flameeyes Exp $
+    $Id: iso9660_fs.c,v 1.43 2007/09/26 05:30:15 rocky Exp $
 
     Copyright (C) 2001 Herbert Valerio Riedel <hvr@gnu.org>
-    Copyright (C) 2003, 2004, 2005, 2006 Rocky Bernstein <rocky@panix.com>
+    Copyright (C) 2003, 2004, 2005, 2006, 2007 Rocky Bernstein <rocky@gnu.org>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -50,7 +50,7 @@
 
 #include <stdio.h>
 
-static const char _rcsid[] = "$Id: iso9660_fs.c,v 1.37 2006/09/26 22:21:09 flameeyes Exp $";
+static const char _rcsid[] = "$Id: iso9660_fs.c,v 1.43 2007/09/26 05:30:15 rocky Exp $";
 
 /* Implementation of iso9660_t type */
 struct _iso9660_s {
@@ -193,7 +193,10 @@ iso9660_open_ext_private (const char *psz_path,
   return p_iso;
 
  error:
-  free(p_iso);
+  if (p_iso && p_iso->stream) {
+    cdio_stdio_destroy(p_iso->stream);
+    free(p_iso);
+  }
   return NULL;
 }
 
@@ -958,6 +961,9 @@ _fs_stat_traverse (const CdIo_t *p_cdio, const iso9660_stat_t *_root,
       unsigned int len=sizeof(iso9660_stat_t) + strlen(_root->filename)+1;
       p_stat = calloc(1, len);
       memcpy(p_stat, _root, len);
+      p_stat->rr.psz_symlink = calloc(1, p_stat->rr.i_symlink_max);
+      memcpy(p_stat->rr.psz_symlink, _root->rr.psz_symlink, 
+	     p_stat->rr.i_symlink_max);
       return p_stat;
     }
 
@@ -996,7 +1002,8 @@ _fs_stat_traverse (const CdIo_t *p_cdio, const iso9660_stat_t *_root,
 
       cmp = strcmp(splitpath[0], p_stat->filename);
 
-      if ( 0 != cmp && 0 == p_env->i_joliet_level && yep != p_stat->rr.b3_rock ) {
+      if ( 0 != cmp && 0 == p_env->i_joliet_level 
+	   && yep != p_stat->rr.b3_rock ) {
 	char *trans_fname = NULL;
 	unsigned int i_trans_fname=strlen(p_stat->filename);
 	int trans_len;
@@ -1051,6 +1058,9 @@ _fs_iso_stat_traverse (iso9660_t *p_iso, const iso9660_stat_t *_root,
       unsigned int len=sizeof(iso9660_stat_t) + strlen(_root->filename)+1;
       p_stat = calloc(1, len);
       memcpy(p_stat, _root, len);
+      p_stat->rr.psz_symlink = calloc(1, p_stat->rr.i_symlink_max);
+      memcpy(p_stat->rr.psz_symlink, _root->rr.psz_symlink, 
+	     p_stat->rr.i_symlink_max);
       return p_stat;
     }
 
@@ -1090,18 +1100,22 @@ _fs_iso_stat_traverse (iso9660_t *p_iso, const iso9660_stat_t *_root,
 
       if ( 0 != cmp && 0 == p_iso->i_joliet_level 
 	   && yep != p_stat->rr.b3_rock ) {
-	char *trans_fname = malloc(strlen(p_stat->filename)+1);
+	char *trans_fname = NULL;
+	unsigned int i_trans_fname=strlen(p_stat->filename);
 	int trans_len;
 	
-	if (trans_fname == NULL) {
-	  cdio_warn("can't allocate %lu bytes", 
-		    (long unsigned int) strlen(p_stat->filename));
-	  return NULL;
+	if (i_trans_fname) {
+	  trans_fname = calloc(1, i_trans_fname+1);
+	  if (!trans_fname) {
+	    cdio_warn("can't allocate %lu bytes", 
+		      (long unsigned int) strlen(p_stat->filename));
+	    return NULL;
+	  }
+	  trans_len = iso9660_name_translate_ext(p_stat->filename, trans_fname, 
+						 p_iso->i_joliet_level);
+	  cmp = strcmp(splitpath[0], trans_fname);
+	  free(trans_fname);
 	}
-	trans_len = iso9660_name_translate_ext(p_stat->filename, trans_fname, 
-					       p_iso->i_joliet_level);
-	cmp = strcmp(splitpath[0], trans_fname);
-	free(trans_fname);
       }
       
       if (!cmp) {
@@ -1151,6 +1165,48 @@ iso9660_fs_stat (CdIo_t *p_cdio, const char psz_path[])
   return p_stat;
 }
 
+typedef iso9660_stat_t * (stat_root_t) (void *p_image);
+typedef iso9660_stat_t * (stat_traverse_t)
+  (const void *p_image, const iso9660_stat_t *_root, char **splitpath);
+
+/*!
+  Get file status for psz_path into stat. NULL is returned on error.
+  pathname version numbers in the ISO 9660
+  name are dropped, i.e. ;1 is removed and if level 1 ISO-9660 names
+  are lowercased.
+ */
+static iso9660_stat_t *
+fs_stat_translate (void *p_image, stat_root_t stat_root, 
+		   stat_traverse_t stat_traverse,
+		   const char psz_path[])
+{
+  iso9660_stat_t *p_root;
+  char **p_psz_splitpath;
+  iso9660_stat_t *p_stat;
+
+  if (!p_image)  return NULL;
+  if (!psz_path) return NULL;
+
+  p_root = stat_root (p_image);
+  if (!p_root) return NULL;
+
+  p_psz_splitpath = _cdio_strsplit (psz_path, '/');
+  p_stat = stat_traverse (p_image, p_root, p_psz_splitpath);
+  free(p_root);
+  _cdio_strfreev (p_psz_splitpath);
+
+  return p_stat;
+}
+
+iso9660_stat_t *
+iso9660_fs_stat_translate (CdIo_t *p_cdio, const char psz_path[], 
+			   bool b_mode2)
+{
+  return fs_stat_translate(p_cdio, (stat_root_t *) _fs_stat_root, 
+			   (stat_traverse_t *) _fs_stat_traverse,
+			   psz_path);
+}
+
 /*!
   Get file status for psz_path into stat. NULL is returned on error.
   pathname version numbers in the ISO 9660
@@ -1158,27 +1214,14 @@ iso9660_fs_stat (CdIo_t *p_cdio, const char psz_path[])
   are lowercased.
  */
 iso9660_stat_t *
-iso9660_fs_stat_translate (CdIo_t *p_cdio, const char psz_path[], 
-			   bool b_mode2)
+iso9660_ifs_stat_translate (iso9660_t *p_iso, const char psz_path[])
 {
-  iso9660_stat_t *p_root;
-  char **p_psz_splitpath;
-  iso9660_stat_t *p_stat;
-
-  if (!p_cdio)  return NULL;
-  if (psz_path) return NULL;
-
-  p_root = _fs_stat_root (p_cdio);
-  if (!p_root) return NULL;
-
-  p_psz_splitpath = _cdio_strsplit (psz_path, '/');
-  p_stat = _fs_stat_traverse (p_cdio, p_root, p_psz_splitpath);
-  free(p_root);
-  _cdio_strfreev (p_psz_splitpath);
-
-  return p_stat;
+  return fs_stat_translate(p_iso, (stat_root_t *) _ifs_stat_root, 
+			   (stat_traverse_t *) _fs_iso_stat_traverse,
+			   psz_path);
 }
 
+
 /*!
   Get file status for psz_path into stat. NULL is returned on error.
  */
@@ -1203,33 +1246,6 @@ iso9660_ifs_stat (iso9660_t *p_iso, const char psz_path[])
   return stat;
 }
 
-/*!
-  Get file status for psz_path into stat. NULL is returned on error.
-  pathname version numbers in the ISO 9660
-  name are dropped, i.e. ;1 is removed and if level 1 ISO-9660 names
-  are lowercased.
- */
-iso9660_stat_t *
-iso9660_ifs_stat_translate (iso9660_t *p_iso, const char psz_path[])
-{
-  iso9660_stat_t *p_root;
-  char **p_psz_splitpath;
-  iso9660_stat_t *p_stat;
-
-  if (!p_iso)    return NULL;
-  if (!psz_path) return NULL;
-
-  p_root = _ifs_stat_root (p_iso);
-  if (NULL == p_root) return NULL;
-
-  p_psz_splitpath = _cdio_strsplit (psz_path, '/');
-  p_stat = _fs_iso_stat_traverse (p_iso, p_root, p_psz_splitpath);
-  free(p_root);
-  _cdio_strfreev (p_psz_splitpath);
-
-  return p_stat;
-}
-
 /*! 
   Read psz_path (a directory) and return a list of iso9660_stat_t
   of the files inside that. The caller must free the returned result.
@@ -1372,10 +1388,15 @@ iso9660_ifs_readdir (iso9660_t *p_iso, const char psz_path[])
   }
 }
 
+typedef CdioList_t * (iso9660_readdir_t) 
+  (void *p_image,  const char * psz_path);
+
 static iso9660_stat_t *
-find_fs_lsn_recurse (CdIo_t *p_cdio, const char psz_path[], lsn_t lsn)
+find_lsn_recurse (void *p_image, iso9660_readdir_t iso9660_readdir,
+		  const char psz_path[], lsn_t lsn,
+		  /*out*/ char **ppsz_full_filename)
 {
-  CdioList_t *entlist = iso9660_fs_readdir (p_cdio, psz_path, true);
+  CdioList_t *entlist = iso9660_readdir (p_image, psz_path);
   CdioList_t *dirlist =  _cdio_list_new ();
   CdioListNode_t *entnode;
     
@@ -1386,15 +1407,18 @@ find_fs_lsn_recurse (CdIo_t *p_cdio, const char psz_path[], lsn_t lsn)
   _CDIO_LIST_FOREACH (entnode, entlist)
     {
       iso9660_stat_t *statbuf = _cdio_list_node_data (entnode);
-      char _fullname[4096] = { 0, };
-      char *filename = (char *) statbuf->filename;
-
-      snprintf (_fullname, sizeof (_fullname), "%s%s/", psz_path, filename);
+      const char *psz_filename  = (char *) statbuf->filename;
+      const unsigned int len = strlen(psz_path) + strlen(psz_filename)+2;
+      
+      if (*ppsz_full_filename != NULL) free(*ppsz_full_filename);
+      *ppsz_full_filename = calloc(1, len);
+      snprintf (*ppsz_full_filename, len, "%s%s/", psz_path, psz_filename);
 
       if (statbuf->type == _STAT_DIR
           && strcmp ((char *) statbuf->filename, ".") 
-          && strcmp ((char *) statbuf->filename, ".."))
-        _cdio_list_append (dirlist, strdup (_fullname));
+          && strcmp ((char *) statbuf->filename, "..")) {
+        _cdio_list_append (dirlist, strdup(*ppsz_full_filename));
+      }
 
       if (statbuf->lsn == lsn) {
 	unsigned int len=sizeof(iso9660_stat_t)+strlen(statbuf->filename)+1;
@@ -1413,62 +1437,13 @@ find_fs_lsn_recurse (CdIo_t *p_cdio, const char psz_path[], lsn_t lsn)
 
   _CDIO_LIST_FOREACH (entnode, dirlist)
     {
-      char *_fullname = _cdio_list_node_data (entnode);
-      iso9660_stat_t *ret_stat = find_fs_lsn_recurse (p_cdio, _fullname, lsn);
-
-      if (NULL != ret_stat) {
-        _cdio_list_free (dirlist, true);
-        return ret_stat;
-      }
-    }
-
-  _cdio_list_free (dirlist, true);
-  return NULL;
-}
-
-static iso9660_stat_t *
-find_ifs_lsn_recurse (iso9660_t *p_iso, const char psz_path[], lsn_t lsn)
-{
-  CdioList_t *entlist = iso9660_ifs_readdir (p_iso, psz_path);
-  CdioList_t *dirlist =  _cdio_list_new ();
-  CdioListNode_t *entnode;
-    
-  cdio_assert (entlist != NULL);
-
-  /* iterate over each entry in the directory */
-  
-  _CDIO_LIST_FOREACH (entnode, entlist)
-    {
-      iso9660_stat_t *statbuf = _cdio_list_node_data (entnode);
-      char _fullname[4096] = { 0, };
-      char *filename = (char *) statbuf->filename;
-
-      snprintf (_fullname, sizeof (_fullname), "%s%s/", psz_path, filename);
-
-      if (statbuf->type == _STAT_DIR
-          && strcmp ((char *) statbuf->filename, ".") 
-          && strcmp ((char *) statbuf->filename, ".."))
-        _cdio_list_append (dirlist, strdup (_fullname));
-
-      if (statbuf->lsn == lsn) {
-	unsigned int len=sizeof(iso9660_stat_t)+strlen(statbuf->filename)+1;
-	iso9660_stat_t *ret_stat = calloc(1, len);
-	memcpy(ret_stat, statbuf, len);
-        _cdio_list_free (entlist, true);
-        _cdio_list_free (dirlist, true);
-        return ret_stat;
-      }
-      
-    }
-
-  _cdio_list_free (entlist, true);
-
-  /* now recurse/descend over directories encountered */
-
-  _CDIO_LIST_FOREACH (entnode, dirlist)
-    {
-      char *_fullname = _cdio_list_node_data (entnode);
-      iso9660_stat_t *ret_stat = find_ifs_lsn_recurse (p_iso, _fullname, lsn);
+      char *psz_path_prefix = _cdio_list_node_data (entnode);
+      iso9660_stat_t *ret_stat;
+      free(*ppsz_full_filename);
+      *ppsz_full_filename = NULL;
+      ret_stat = find_lsn_recurse (p_image, iso9660_readdir, 
+				   psz_path_prefix, lsn,
+				   ppsz_full_filename);
 
       if (NULL != ret_stat) {
         _cdio_list_free (dirlist, true);
@@ -1476,6 +1451,10 @@ find_ifs_lsn_recurse (iso9660_t *p_iso, const char psz_path[], lsn_t lsn)
       }
     }
 
+  if (*ppsz_full_filename != NULL) {      
+    free(*ppsz_full_filename);
+    *ppsz_full_filename = NULL;
+  }
   _cdio_list_free (dirlist, true);
   return NULL;
 }
@@ -1489,7 +1468,23 @@ find_ifs_lsn_recurse (iso9660_t *p_iso, const char psz_path[], lsn_t lsn)
 iso9660_stat_t *
 iso9660_fs_find_lsn(CdIo_t *p_cdio, lsn_t i_lsn)
 {
-  return find_fs_lsn_recurse (p_cdio, "/", i_lsn);
+  char *psz_full_filename = NULL;
+  return find_lsn_recurse (p_cdio, (iso9660_readdir_t *) iso9660_fs_readdir, 
+			   "/", i_lsn, &psz_full_filename);
+}
+
+/*!
+   Given a directory pointer, find the filesystem entry that contains
+   lsn and return information about it.
+
+   Returns stat_t of entry if we found lsn, or NULL otherwise.
+ */
+iso9660_stat_t *
+iso9660_fs_find_lsn_with_path(CdIo_t *p_cdio, lsn_t i_lsn,
+			      /*out*/ char **ppsz_full_filename)
+{
+  return find_lsn_recurse (p_cdio, (iso9660_readdir_t *) iso9660_fs_readdir, 
+			   "/", i_lsn, ppsz_full_filename);
 }
 
 /*!
@@ -1501,7 +1496,23 @@ iso9660_fs_find_lsn(CdIo_t *p_cdio, lsn_t i_lsn)
 iso9660_stat_t *
 iso9660_ifs_find_lsn(iso9660_t *p_iso, lsn_t i_lsn)
 {
-  return find_ifs_lsn_recurse (p_iso, "/", i_lsn);
+  char *psz_full_filename = NULL;
+  return find_lsn_recurse (p_iso, (iso9660_readdir_t *) iso9660_ifs_readdir, 
+			   "/", i_lsn, &psz_full_filename);
+}
+
+/*!
+   Given a directory pointer, find the filesystem entry that contains
+   lsn and return information about it.
+
+   Returns stat_t of entry if we found lsn, or NULL otherwise.
+ */
+iso9660_stat_t *
+iso9660_ifs_find_lsn_with_path(iso9660_t *p_iso, lsn_t i_lsn,
+			       /*out*/ char **ppsz_full_filename)
+{
+  return find_lsn_recurse (p_iso, (iso9660_readdir_t *) iso9660_ifs_readdir, 
+			   "/", i_lsn, ppsz_full_filename);
 }
 
 /*!
