@@ -3,7 +3,7 @@
 # Tests to see that BIN/CUE and cdrdao TOC file iamge reading is correct
 # (via cd-info).
 
-if test "-L/usr/local/lib -lvcdinfo -liso9660 -lcdio -lm  X" != "X" ; then
+if test "X" != "X" ; then
   vcd_opt='--no-vcd'
 fi
 
@@ -73,7 +73,7 @@ if test -n "1"; then
 fi
 
 fname=vcd_demo
-if test -z "-L/usr/local/lib -lvcdinfo -liso9660 -lcdio -lm  " ; then
+if test -z "" ; then
   right=${srcdir}/${fname}.right
 else
   right=${srcdir}/${fname}_vcdinfo.right
@@ -85,7 +85,7 @@ if test -f ${srcdir}/${fname}.bin ; then
   RC=$?
   check_result $RC "cd-info CUE test $testnum" "${CD_INFO} $opts"
 
-  if test -z "-L/usr/local/lib -lvcdinfo -liso9660 -lcdio -lm  " ; then
+  if test -z "" ; then
     right=${srcdir}/${fname}.right
   else
     right=${srcdir}/${fname}_vcdinfo.right
